﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Activities;
using Microsoft.Xrm.Sdk;
using Microsoft.Xrm.Sdk.Workflow;

/**
 * Copyright 2011 by Alberto Gemin
 * agemin@hotmail.com
 * Version 1.2.0.1
 * 19 Dec 2011
 **/
namespace Gemina.CRM2011.WF.CrmDistributeWF
{

    /// <summary>
    /// Custom base class for all Code Activities.
    /// </summary>
    public abstract class CodeActivityBase : CodeActivity
    {

        #region Properties

        /// <summary>
        /// Primary Key of the entity the WF runs on.
        /// </summary>
        public Guid CurrentRecordId { get; set; }

        /// <summary>
        /// Name of the entity the WF runs on.
        /// </summary>
        public string CurrentEntityName { get; set; }

        /// <summary>
        /// Operation that triggered the workflow.
        /// </summary>
        public string CurrentOperation { get; set; }

        #endregion

        #region CodeActivity

        /// <summary>
        /// Workflow step.
        /// </summary>
        protected override void Execute(CodeActivityContext executionContext)
        {
            try
            {
                IWorkflowContext workflowContext = executionContext.GetExtension<IWorkflowContext>();
                this.CurrentRecordId = workflowContext.PrimaryEntityId;
                this.CurrentOperation = workflowContext.MessageName;
                this.CurrentEntityName = workflowContext.PrimaryEntityName;

                this.ExecuteBody(executionContext);
            }
            catch (AgErrorMessage ex)
            {
                throw new InvalidPluginExecutionException(OperationStatus.Failed, ex.Message);
            }
            catch (Exception ex)
            {
                throw new InvalidPluginExecutionException(OperationStatus.Failed, ex.ToString());
            }
        }

        /// <summary>
        /// Overridden in derived classes: contains the actual workflow step.
        /// </summary>
        protected abstract bool ExecuteBody(CodeActivityContext executionContext);

        #endregion

        #region Protected Methods

        protected IOrganizationService GetService(CodeActivityContext executionContext, Guid? user)
        {
            IOrganizationServiceFactory svcFactory = executionContext.GetExtension<IOrganizationServiceFactory>();

            if (user == null)
            {
                IWorkflowContext workflowContext = executionContext.GetExtension<IWorkflowContext>();
                return svcFactory.CreateOrganizationService(workflowContext.UserId);
            }
            else
            {
                return svcFactory.CreateOrganizationService(user);
            }
        }

        protected IOrganizationService GetService(CodeActivityContext executionContext)
        {
            return this.GetService(executionContext, null);
        }

        #endregion
    }

}
