﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xrm.Sdk.Workflow;
using System.Activities;
using Microsoft.Xrm.Sdk;
using Microsoft.Crm.Sdk.Messages;

/**
 * Copyright 2011 by Alberto Gemin
 * agemin@hotmail.com
 * Version 1.2.0.1
 * 19 Dec 2011
 **/
namespace Gemina.CRM2011.WF.CrmDistributeWF
{
    abstract public class DistributeWFActivity : CodeActivityBase
    {

        #region Workflow Parameters

        /// <summary>
        /// Input parameter: Workflow to be executed.
        /// </summary>
        [Input("Distributed Workflow")]
        [ReferenceTarget("workflow")]
        [RequiredArgument]
        public InArgument<EntityReference> Workflow { get; set; }

        /// <summary>
        /// Input parameter: Workflow to be executed.
        /// </summary>
        [Input("Relationship Name")]
        [RequiredArgument]
        public InArgument<String> RelationshipName { get; set; }

        #endregion

        #region CodeActivity

        override protected bool ExecuteBody(CodeActivityContext executionContext)
        {
            this.Distribute(executionContext);

            return true;
        }

        #endregion

        #region Protected Methods

        /// <summary>
        /// Launch workflows for dependent entities.
        /// </summary>
        /// <param name="keyList">List id IDs of dependent entities</param>
        /// <param name="workflowId">Id of workflow to execute</param>
        protected void Distribute(CodeActivityContext executionContext)
        {
            List<Guid> keyList = this.GatherKeys(executionContext);
            Guid workflowId = this.Workflow.Get(executionContext).Id;

            foreach (Guid key in keyList)
            {
                ExecuteWorkflowRequest workflowRequest = new ExecuteWorkflowRequest();
                workflowRequest.EntityId = key;
                workflowRequest.WorkflowId = workflowId;
                this.GetService(executionContext).Execute(workflowRequest);
            }
        }

        /// <summary>
        /// Get IDs of dependent entities
        /// </summary>
        protected abstract List<Guid> GatherKeys(CodeActivityContext executionContext);

        #endregion

    }
}
